@@ -33,8 +33,6 @@ char *mnt_path = "/tmp/partclone_nilfs2_mnt/"; //fixme
 struct nilfs_super_block *sbp;
 struct nilfs *nilfs;
 
-char *EXECNAME = "partclone.nilfs";
-
 extern fs_cmd_opt fs_opt;
 
 static struct nilfs_suinfo suinfos[LSSU_NSEGS];
@@ -59,7 +57,7 @@ static void set_bitmap(unsigned long* bitmap, uint64_t segm, uint64_t count){
 
     for(block = pos_block; block < block_end; block++){
 	log_mesg(3, 0, 0, fs_opt.debug, "%s: block %i is used\n",__FILE__,  block);
-	pc_set_bit(block, bitmap);
+	pc_set_bit(block, bitmap, total_block);
     }
 }
 
@@ -148,7 +146,7 @@ static void fs_close()
 }
 
 ///  readbitmap - read bitmap
-extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap, int pui)
+extern void read_bitmap(char* device, file_system_info fs_info, unsigned long* bitmap, int pui)
 {
     int start = 0;
     int bit_size = 1;
@@ -157,10 +155,10 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
     fs_open(device);
     /// init progress
     progress_bar   prog;	/// progress_bar structure defined in progress.h
-    progress_init(&prog, start, image_hdr.totalblock, image_hdr.totalblock, BITMAP, bit_size);
+    progress_init(&prog, start, fs_info.totalblock, fs_info.totalblock, BITMAP, bit_size);
 
     blocks_per_segment = nilfs_get_blocks_per_segment(nilfs);
-    total_block = image_hdr.totalblock;
+    total_block = fs_info.totalblock;
     status = lssu_list_suinfo(nilfs, bitmap);
 
     if (status == 1 ){
@@ -173,16 +171,16 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 }
 
 /// read super block and write to image head
-extern void initial_image_hdr(char* device, image_head* image_hdr)
+extern void read_super_blocks(char* device, file_system_info* fs_info)
 {
     fs_open(device);
     sbp = nilfs->n_sb;
-    strncpy(image_hdr->magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE);
-    strncpy(image_hdr->fs, nilfs_MAGIC, FS_MAGIC_SIZE);
-    image_hdr->block_size  = nilfs_get_block_size(nilfs);
-    image_hdr->totalblock  = sbp->s_dev_size / image_hdr->block_size;
-    image_hdr->usedblocks  = image_hdr->totalblock - sbp->s_free_blocks_count;
-    image_hdr->device_size = sbp->s_dev_size;
+    //strncpy(fs_info->magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE);
+    strncpy(fs_info->fs, nilfs_MAGIC, FS_MAGIC_SIZE);
+    fs_info->block_size  = nilfs_get_block_size(nilfs);
+    fs_info->totalblock  = sbp->s_dev_size / fs_info->block_size;
+    fs_info->usedblocks  = fs_info->totalblock - sbp->s_free_blocks_count;
+    fs_info->device_size = sbp->s_dev_size;
     fs_close();
 }
 
