@@ -32,9 +32,6 @@
 
 dal_t		 *dal;
 reiserfs_fs_t	 *fs;
-char *EXECNAME = "partclone.reiserfs";
-
-extern fs_cmd_opt fs_opt;
 
 /// open device
 static void fs_open(char* device){
@@ -64,8 +61,7 @@ static void fs_close(){
 
 }
 
-///  readbitmap - read bitmap
-extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap, int pui)
+void read_bitmap(char* device, file_system_info fs_info, unsigned long* bitmap, int pui)
 {
     reiserfs_bitmap_t    *fs_bitmap;
     reiserfs_tree_t	 *tree;
@@ -89,10 +85,10 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 	log_mesg(3, 0, 0, fs_opt.debug, "%s: block bitmap check %llu\n", __FILE__, blk);
 	if(reiserfs_tools_test_bit(blk, fs_bitmap->bm_map)){
 	    bused++;
-	    pc_set_bit(blk, bitmap);
+	    pc_set_bit(blk, bitmap, fs_info.totalblock);
 	}else{
 	    bfree++;
-	    pc_clear_bit(blk, bitmap);
+	    pc_clear_bit(blk, bitmap, fs_info.totalblock);
 	}
 	/// update progress
 	update_pui(&bprog, blk, blk, done);
@@ -108,16 +104,14 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 
 }
 
-/// read super block and write to image head
-extern void initial_image_hdr(char* device, image_head* image_hdr)
+void read_super_blocks(char* device, file_system_info* fs_info)
 {
     fs_open(device);
-    strncpy(image_hdr->magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE);
-    strncpy(image_hdr->fs, reiserfs_MAGIC, FS_MAGIC_SIZE);
-    image_hdr->block_size = (int)fs->super->s_v1.sb_block_size;
-    image_hdr->totalblock = (unsigned long long)fs->super->s_v1.sb_block_count;
-    image_hdr->usedblocks = (unsigned long long)(fs->super->s_v1.sb_block_count - fs->super->s_v1.sb_free_blocks);
-    image_hdr->device_size = (unsigned long long)(image_hdr->block_size * image_hdr->totalblock);
+    strncpy(fs_info->fs, reiserfs_MAGIC, FS_MAGIC_SIZE);
+    fs_info->block_size  = fs->super->s_v1.sb_block_size;
+    fs_info->totalblock  = fs->super->s_v1.sb_block_count;
+    fs_info->usedblocks  = fs->super->s_v1.sb_block_count - fs->super->s_v1.sb_free_blocks;
+    fs_info->device_size = fs_info->block_size * fs_info->totalblock;
     fs_close();
 }
 
