@@ -39,9 +39,6 @@ struct uufsd disk;
 #include "progress.h"
 #include "fs_common.h"
 
-char *EXECNAME = "partclone.ufs";
-extern fs_cmd_opt fs_opt;
-
 /// get_used_block - get FAT used blocks
 static unsigned long long get_used_block()
 {
@@ -134,8 +131,7 @@ static void fs_close(){
     log_mesg(1, 0, 0, fs_opt.debug, "%s: done\n\n", __FILE__);
 }
 
-/// readbitmap - read bitmap
-extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap, int pui)
+void read_bitmap(char* device, file_system_info fs_info, unsigned long* bitmap, int pui)
 {
     unsigned long long     total_block, block, bused = 0, bfree = 0;
     int                    done = 0, i = 0, start = 0, bit_size = 1;
@@ -146,7 +142,7 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 
     /// init progress
     progress_bar   bprog;	/// progress_bar structure defined in progress.h
-    progress_init(&bprog, start, image_hdr.totalblock, image_hdr.totalblock, BITMAP, bit_size);
+    progress_init(&bprog, start, fs_info.totalblock, fs_info.totalblock, BITMAP, bit_size);
 
     total_block = 0;
     /// read group
@@ -157,11 +153,11 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 
         for (block = 0; block < acg.cg_ndblk; block++){
             if (isset(p, block)) {
-                pc_clear_bit(total_block, bitmap, image_hdr.totalblock);
+                pc_clear_bit(total_block, bitmap, fs_info.totalblock);
                 bfree++;
                 log_mesg(3, 0, 0, fs_opt.debug, "%s: bitmap is free %lli\n", __FILE__, block);
             } else {
-                pc_set_bit(total_block, bitmap, image_hdr.totalblock);
+                pc_set_bit(total_block, bitmap, fs_info.totalblock);
                 bused++;
                 log_mesg(3, 0, 0, fs_opt.debug, "%s: bitmap is used %lli\n", __FILE__, block);
             }
@@ -179,25 +175,24 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 
 }
 
-/// read super block and write to image head
-extern void initial_image_hdr(char* device, image_head* image_hdr)
+void read_super_blocks(char* device, file_system_info* fs_info)
 {
 
     fs_open(device);
-    strncpy(image_hdr->magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE);
-    strncpy(image_hdr->fs, ufs_MAGIC, FS_MAGIC_SIZE);
-    image_hdr->block_size  = afs.fs_fsize;
-    image_hdr->usedblocks  = get_used_block();
+    strncpy(fs_info->fs, ufs_MAGIC, FS_MAGIC_SIZE);
+    fs_info->block_size = afs.fs_fsize;
+    fs_info->usedblocks = get_used_block();
     switch (disk.d_ufs) {
         case 2:
-	    image_hdr->totalblock = (unsigned long long)afs.fs_size;
-            image_hdr->device_size = afs.fs_fsize*afs.fs_size;
+            fs_info->totalblock  = afs.fs_size;
+            fs_info->device_size = afs.fs_fsize*afs.fs_size;
             break;
         case 1:
-	    image_hdr->totalblock = (unsigned long long)afs.fs_old_size;
-            image_hdr->device_size = afs.fs_fsize*afs.fs_old_size;
+            fs_info->totalblock  = afs.fs_old_size;
+            fs_info->device_size = afs.fs_fsize*afs.fs_old_size;
             break;
         default:
+            log_mesg(0, 1, 1, fs_opt.debug, "Warning: unknown ufs's version [%d]", disk.d_ufs);
             break;
     }
 
