@@ -26,8 +26,6 @@
 
 vmfs_fs_t *fs;
 vmfs_dir_t *root_dir;
-char *EXECNAME = "partclone.vmfs";
-extern fs_cmd_opt fs_opt;
 unsigned long *blk_bitmap;
 progress_bar   prog;        /// progress_bar structure defined in progress.h
 unsigned long long checked;
@@ -334,8 +332,7 @@ static void fs_close(){
     vmfs_fs_close(fs);
 }
 
-/// readbitmap - read bitmap
-extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap, int pui)
+void read_bitmap(char* device, file_system_info fs_info, unsigned long* bitmap, int pui)
 {
     unsigned long long used_block = 0, free_block = 0, err_block = 0;
     int start = 0;
@@ -360,7 +357,8 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
     blk_bitmap = bitmap;
 
     /// init progress
-    progress_init(&prog, start, image_hdr.usedblocks, image_hdr.usedblocks, BITMAP, bit_size);
+    progress_init(&prog, start, fs_info.usedblocks, fs_info.usedblocks, BITMAP, bit_size);
+    pc_init_bitmap(bitmap, 0x00, fs_info.totalblock);
     checked = 0;
     /**
      * thread to print progress
@@ -372,9 +370,9 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 
     /// add base
 
-    pc_set_bit(vmfs_hb_base/vmfs_fs_get_blocksize(fs), bitmap, image_hdr.totalblock);
-    pc_set_bit(vmfs_fsinfo_base/vmfs_fs_get_blocksize(fs), bitmap, image_hdr.totalblock);
-    pc_set_bit(vmfs_volinfo_base/vmfs_fs_get_blocksize(fs), bitmap, image_hdr.totalblock);
+    pc_set_bit(vmfs_hb_base/vmfs_fs_get_blocksize(fs), bitmap, fs_info.totalblock);
+    pc_set_bit(vmfs_fsinfo_base/vmfs_fs_get_blocksize(fs), bitmap, fs_info.totalblock);
+    pc_set_bit(vmfs_volinfo_base/vmfs_fs_get_blocksize(fs), bitmap, fs_info.totalblock);
 
     fdc_bmp = &fs->fdc->bmh;
     log_mesg(3, 0, 0, fs_opt.debug, "Scanning %u FDC entries...\n",fdc_bmp->total_items);
@@ -413,16 +411,14 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 
 }
 
-/// read super block and write to image head
-extern void initial_image_hdr(char* device, image_head* image_hdr)
+void read_super_blocks(char* device, file_system_info* fs_info)
 {
 
     uint32_t alloc,total;
     uint32_t fdc_allocated, fbb_allocated, sbc_allocated, pbc_allocated;
 
     fs_open(device);
-    strncpy(image_hdr->magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE);
-    strncpy(image_hdr->fs, vmfs_MAGIC, FS_MAGIC_SIZE);
+    strncpy(fs_info->fs, vmfs_MAGIC, FS_MAGIC_SIZE);
     total = fs->fbb->bmh.total_items;
     fdc_allocated = vmfs_bitmap_allocated_items(fs->fdc);
     fbb_allocated = vmfs_bitmap_allocated_items(fs->fbb);
@@ -434,15 +430,15 @@ extern void initial_image_hdr(char* device, image_head* image_hdr)
     log_mesg(3, 0, 0, fs_opt.debug, "allocated sbc %"PRIu32"\n", sbc_allocated);
     log_mesg(3, 0, 0, fs_opt.debug, "allocated pbc %"PRIu32"\n", pbc_allocated);
 
-    image_hdr->block_size  = vmfs_fs_get_blocksize(fs);
-    image_hdr->totalblock  = total;
+    fs_info->block_size  = vmfs_fs_get_blocksize(fs);
+    fs_info->totalblock  = total;
+    fs_info->usedblocks  = alloc;
+    fs_info->device_size = vmfs_fs_get_blocksize(fs) * total;
     total_block = total+100;
-    image_hdr->usedblocks  = alloc;
-    image_hdr->device_size = (vmfs_fs_get_blocksize(fs)*total);
-    log_mesg(3, 0, 0, fs_opt.debug, "block_size %u\n", image_hdr->block_size);
-    log_mesg(3, 0, 0, fs_opt.debug, "totalblock %llu\n", image_hdr->totalblock);
-    log_mesg(3, 0, 0, fs_opt.debug, "device_size %llu\n",image_hdr->device_size);
-    log_mesg(3, 0, 0, fs_opt.debug, "usedblocks %llu\n", image_hdr->usedblocks);
+    log_mesg(3, 0, 0, fs_opt.debug, "block_size %u\n", fs_info->block_size);
+    log_mesg(3, 0, 0, fs_opt.debug, "totalblock %llu\n", fs_info->totalblock);
+    log_mesg(3, 0, 0, fs_opt.debug, "device_size %llu\n", fs_info->device_size);
+    log_mesg(3, 0, 0, fs_opt.debug, "usedblocks %llu\n", fs_info->usedblocks);
 
     fs_close();
 }
